<?php

require_once('Pencil.php');

/*
* ImageCube originally by Lars Vandenbergh
* 3rd september 2004
* modifications by Michiel van der Blonk, (c) 2006
*
* Copyright (C) 2007 Michiel van der Blonk - blonkm@gmail.com - All Rights Reserved
* Based on C++ code by Jol van Noort
*
* This software is licensed as per the MIT License below:
* MIT License: http://www.opensource.org/licenses/mit-license.php
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
* A PHP script for generating static 3D views of the 3x3x3 Rubik's Cube
*/

class ImageCube
{
  var $im;
  var $drawMode;
  var $pencil;
  var $size;
  var $stickers;
  var $scale;
  var $scalepx;
  var $colors;
  var $back;
  var $format;

  function get($s, $default='')
  {
	if (isset($_GET[$s]))
		return $_GET[$s];
	if (isset($_POST[$s]))
		return $_POST[$s];
	return $default;
  }

  function ImageCube()
  {
    // initialize paramaters
	$this->size = ImageCube::get('size', 200);
	$this->stickers = ImageCube::get('stickers', '');
	$this->drawMode = ImageCube::get('drawMode', 'thick');
    $this->scale = 2;
    $this->scalepx = $this->size * $this->scale / 200;
    $this->im = imagecreate($this->size * $this->scale, $this->size * $this->scale);
    imageantialias ($this->im, true);
    // create pencil object for drawing lines
    $this->pencil = new Pencil;
    $this->pencil->mode = $this->drawMode;
	$this->format = ImageCube::get('format', 'png');
  }

  function prepare()
  {
    $this->setColors();
    $this->drawOutline();
    $this->drawEdges();
    $this->drawStickers();
  }

  function setColors()
  {
    // allocate colors
    $this->back = imageColorAllocate($this->im, 255, 255, 255);
    $this->edgeColor = imageColorAllocate($this->im, 10,10,10);
    $red = imageColorAllocate($this->im, 200, 0, 0);
    $orange = imageColorAllocate($this->im, 255, 161, 0);
    $blue = imageColorAllocate($this->im, 0, 0, 182);
    $green = imageColorAllocate($this->im, 0, 182, 72);
    $white = imageColorAllocate($this->im, 247, 247, 247);
    $yellow = imageColorAllocate($this->im, 239, 239, 0);
    $grey = imageColorAllocate($this->im, 200, 200, 200);
    $this->colors = array("r" => $red,
      "o" => $orange,
      "b" => $blue,
      "g" => $green,
      "w" => $white,
      "y" => $yellow,
      "x" => $grey);
  }

	function drawOutline()
	{
		// draw cube silhouette
		$sx = $this->scalepx;
		$sy = $this->scalepy;
		$c = $this->edgeColor;
		$width = $this->size/100;
		$this->pencil->drawLine($this->im, 78    * $sx, 16   * $sx, 183   * $sx, 28   * $sx, $c, $width);
		$this->pencil->drawLine($this->im, 183  * $sx, 28   * $sx, 185   * $sx, 133 * $sx, $c, $width);
		$this->pencil->drawLine($this->im, 185  * $sx, 133 * $sx, 141   * $sx, 194 * $sx, $c, $width);
		$this->pencil->drawLine($this->im, 141  * $sx, 194 * $sx, 31     * $sx, 168 * $sx, $c, $width);
		$this->pencil->drawLine($this->im, 31    * $sx, 168 * $sx, 15     * $sx, 62   * $sx, $c, $width);
		$this->pencil->drawLine($this->im, 15    * $sx, 62   * $sx, 78     * $sx, 16   * $sx, $c, $width);
	}

  function drawEdge($x, $y, $xEnd, $yEnd)
  {
    $sx = $this->scalepx * $x;
    $sy = $this->scalepx * $y;
    $sxEnd = $this->scalepx * $xEnd;
    $syEnd = $this->scalepx * $yEnd;
    $this->pencil->drawLine($this->im, $sx, $sy, $sxEnd, $syEnd, $this->edgeColor, $this->size/50);
  }

  function drawEdges()
  {
    // draw inner edges

    $this->drawEdge(134, 78, 182, 29);
    $this->drawEdge(134, 78, 141, 193);
    $this->drawEdge(134, 78, 15, 62);

    // draw BEF layers
    $this->drawEdge(60, 29, 168, 43);
    $this->drawEdge(168, 43, 172, 151);
    $this->drawEdge(39, 45, 151, 60);
    $this->drawEdge(151, 60, 158, 170);

    // draw LMR layers
    $this->drawEdge(108, 19, 54, 67);
    $this->drawEdge(54, 67, 65, 176);
    $this->drawEdge(144, 23, 93, 73);
    $this->drawEdge(93, 73, 103, 185);

    // draw UED layers
    $this->drawEdge(21, 100, 136, 119);
    $this->drawEdge(136, 119, 184, 65);
    $this->drawEdge(26, 136, 138, 158);
    $this->drawEdge(138, 158, 184, 102);
  }

  // fill the space of one sticker with a certain color
  function drawSticker($x, $y, $colorIndex)
  {
    $color = $this->colors[substr($this->stickers, $colorIndex, 1)];
    $xPos = $this->scalepx * $x;
    $yPos = $this->scalepx * $y;
 	imagefill($this->im, $xPos, $yPos, $color);
  }

  function drawStickers()
  {
    $this->drawSticker(85, 25, 0);    // UBL
    $this->drawSticker(123, 30, 1);   // UB
    $this->drawSticker(158, 32, 2);   // UBR
    $this->drawSticker(71, 40, 3);    // UL
    $this->drawSticker(101, 40, 4);   // U
    $this->drawSticker(143, 46, 5);   // UR
    $this->drawSticker(43, 56, 6);    // UFL
    $this->drawSticker(88, 57, 7);    // UF
    $this->drawSticker(121, 63, 8);   // UFR
    $this->drawSticker(35, 91, 9);    // FLU
    $this->drawSticker(78, 91, 10);   // FU
    $this->drawSticker(120, 91, 11);    // FRU
    $this->drawSticker(40, 135, 12);    // FL
    $this->drawSticker(80, 135, 13);    // F
    $this->drawSticker(120, 135, 14); // FR
    $this->drawSticker(50, 170, 15);    // FDL
    $this->drawSticker(90, 170, 16);    // FD
    $this->drawSticker(130, 170, 17); // FDR
    $this->drawSticker(146, 87, 18);    // RUF
    $this->drawSticker(162, 68, 19);    // RU
    $this->drawSticker(177, 51, 20);    // RUB
    $this->drawSticker(150, 130, 21); // RF
    $this->drawSticker(160, 110, 22); // R
    $this->drawSticker(180, 95, 23);    // RB
    $this->drawSticker(510, 170, 24); // RFD
    $this->drawSticker(160, 145, 25); // RD
    $this->drawSticker(180, 130, 26); // RBD
  }

  function save()
  {
    // resample image and make background transparent
    $output = imageCreateTrueColor($this->size, $this->size);
    imageCopyResampled($output, $this->im, 0, 0, 0, 0, $this->size, $this->size, $this->size * $this->scale, $this->size * $this->scale);
    $back = imageColorAllocate($output, 255, 255, 255);
    imageColorTransparent($output, $this->back);
    imageFill($output, 0, 0, $this->back);
    // send image to client
	switch (strtolower($this->format))
	{
		case 'gif':
			header('Content-type: image/gif');
			imagegif($this->im);
			break;
		case 'jpg':
		case 'jpeg':
			header('Content-type: image/jpeg');
			imageInterlace($this->im, true);
			imageJpeg($this->im, '', 90);
			break;
		case 'png':
			header('Content-type: image/png');
			imageAlphaBlending( $this->im, false );
			imageSaveAlpha( $this->im, true );
			imagepng($this->im);
			break;
	}
    imagedestroy($this->im);
    imagedestroy($output);
  }
}

// create object and output to stream
$objImageCube = new ImageCube;
$objImageCube->prepare();
$objImageCube->save();
?>