<?php
/*
* This software is licensed as per the MIT License below:
* MIT License: http://www.opensource.org/licenses/mit-license.php
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

	class Pencil {
		# antialiased draw_line function 1.1 (faster)

		# here is a drawLine() posted by nanobot at chipx86 dot com
		# on php.net and enhanced/optimized by myself :)

		# here are some changes i made:
		# 1. changed for true-color images (no index_var used)
		# 2. changed rgb extraction to logical shift
		# 3. reducing function call's

		var $mode;

		function drawLine($image, $x1, $y1, $x2, $y2, $color, $width = 4)
		{
				switch($this->mode)
				{
					case "":
					case "normal":
						imageline($image, $x1, $y1, $x2, $y2, $color);
						break;
					case "antialias":
						$this->drawAntiAliasLine($image, $x1, $y1, $x2, $y2);
						break;
					case "font":
						$this->drawFontLine($image, $x1, $y1, $x2, $y2, $color);
						break;
					case "thick":
						$this->drawThickLine($image, $x1, $y1, $x2, $y2, $color, $width);
						break;
				}
		}

		function drawAntiAliasLine ($image, $x1, $y1, $x2, $y2, $r=0, $g=0, $b=0) {
			$icr = $r;
			$icg = $g;
			$icb = $b;
			$dcol = ImageColorAllocate ($image, $icr, $icg, $icb);
			if ($y1 == $y2) imageline ($image, $x1, $y1, $x2, $y1, $dcol);
				else if ($x1 == $x2) {
				imageline ($image, $x1, $y1, $x1, $y2, $dcol);
			} else {
				$m = ($y2 - $y1) / ($x2 - $x1);
				$b = $y1 - $m * $x1;

				if (abs ($m) < 2) {

					$x = min ($x1, $x2);
					$endx = max ($x1, $x2)+1;

					while ($x < $endx) {
						$y = $m * $x + $b;
						$y == floor ($y) ? $ya = 1 :
						 $ya = $y - floor ($y);
						$yb = ceil ($y) - $y;

						$trgb = ImageColorAt($image, $x, floor($y));
						$tcr = ($trgb >> 16) & 0xFF;
						$tcg = ($trgb >> 8) & 0xFF;
						$tcb = $trgb & 0xFF;
						imagesetpixel ($image, $x, floor ($y), imagecolorallocate ($image, ($tcr * $ya + $icr * $yb),
							($tcg * $ya + $icg * $yb),
							($tcb * $ya + $icb * $yb)));

						$trgb = ImageColorAt($image, $x, ceil($y));
						$tcr = ($trgb >> 16) & 0xFF;
						$tcg = ($trgb >> 8) & 0xFF;
						$tcb = $trgb & 0xFF;
						imagesetpixel ($image, $x, ceil ($y), imagecolorallocate ($image, ($tcr * $yb + $icr * $ya),
							($tcg * $yb + $icg * $ya),
							($tcb * $yb + $icb * $ya)));

						$x ++;
					}
					# while_x_end
				}
				# if_end
				else
				{
					# else_abs_end

					$y = min ($y1, $y2);
					$endy = max ($y1, $y2)+1;

					while ($y < $endy) {
						$x = ($y - $b) / $m;
						$x == floor ($x) ? $xa = 1 :
						 $xa = $x - floor ($x);
						$xb = ceil ($x) - $x;

						$trgb = ImageColorAt($image, floor($x), $y);
						$tcr = ($trgb >> 16) & 0xFF;
						$tcg = ($trgb >> 8) & 0xFF;
						$tcb = $trgb & 0xFF;
						imagesetpixel ($image, floor ($x), $y, imagecolorallocate ($image, ($tcr * $xa + $icr * $xb),
							($tcg * $xa + $icg * $xb),
							($tcb * $xa + $icb * $xb)));

						$trgb = ImageColorAt($image, ceil($x), $y);
						$tcr = ($trgb >> 16) & 0xFF;
						$tcg = ($trgb >> 8) & 0xFF;
						$tcb = $trgb & 0xFF;
						imagesetpixel ($image, ceil ($x), $y, imagecolorallocate ($image, ($tcr * $xb + $icr * $xa),
							($tcg * $xb + $icg * $xa),
							($tcb * $xb + $icb * $xa)));

						$y ++;
					}
					# while_y_end
				}
				# else_abs_end
			}
			# else_y=y_x=x_end
		}
		# drawOSLine_end

		/* MB: other version */
		# antialiased lines with true-type-fonts !?
		# yip, it works - not very good - but faster
		# (below 3 sec) than other routines (~7 sec)
		# i saw here.

		# the antialiased effect is minimal and only
		# at the top of the line - but, as better as
		# nothing ;)

		# here is a drawLine routine from Logan Gunthorpe
		# (logang at deltatee dot com) posted by nanobot at chipx86 dot com
		# on php.net and enhanced/optimized by myself :)

		function drawFontLine ($image, $x1, $y1, $x2, $y2, $r=0,$g=0,$b=0, $size=3) {

			$fontfile="arial.ttf"; # the font for the dot
			$str="?" ; # the point_char for the dot
			$size=3; # the point for the dot_size

			 $color = ImageColorAllocate ($image, $r,$g,$b);

			 if ($x2 == $x1) {
				 $tmp = $x1; $x1 = $y1; $y1 = $tmp;
				 $tmp = $x2; $x2 = $y2; $y2 = $tmp;
				 $swapx = true;
			 }

			 $m = (float)(( $y2 - $y1 ) / ( $x2 - $x1 ));
			 $b = (float)($y1 - $m*$x1);

			 $strx = min ( $x1 , $x2 );
			 $endx = max ( $x1 , $x2 );
			 $x=(int)0;

			# the original loop was based on one for_loop
			# with the if_swap inside. to speed up this loop
			# i double it, to keep off the permanent if_call's.

			if (!$swapx) {
			 for ($x = $strx; $x <= $endx; $x++) {
			   $y = (int)($m*$x + $b);
			   imagettftext($image, $size, 0, $x, $y,
			   $color, $fontfile,$str);
			 }
			} else {
			 for ($x = $strx; $x <= $endx; $x++) {
			   $y = (int)($m*$x + $b);
			   imagettftext($image, $size, 0, $y, $x,
			   $color, $fontfile,$str);
			  }
			}

		}

		/* another one for thick lines */
		function drawThickLine($image, $x1, $y1, $x2, $y2, $color, $thick = 1)
		{
		   /* this way it works well only for orthogonal lines
		   imagesetthickness($image, $thick);
		   return imageline($image, $x1, $y1, $x2, $y2, $color);
		   */
		   if ($thick == 1) {
			   return imageline($image, $x1, $y1, $x2, $y2, $color);
		   }
		   $t = $thick / 2 - 0.5;
		   if ($x1 == $x2 || $y1 == $y2) {
			   return imagefilledrectangle($image, round(min($x1, $x2) - $t), round(min($y1, $y2) - $t), round(max($x1, $x2) + $t), round(max($y1, $y2) + $t), $color);
		   }
		   $k = ($y2 - $y1) / ($x2 - $x1); //y = kx + q
		   $a = $t / sqrt(1 + pow($k, 2));
		   $points = array(
			   round($x1 - (1+$k)*$a), round($y1 + (1-$k)*$a),
			   round($x1 - (1-$k)*$a), round($y1 - (1+$k)*$a),
			   round($x2 + (1+$k)*$a), round($y2 - (1-$k)*$a),
			   round($x2 + (1-$k)*$a), round($y2 + (1+$k)*$a),
		   );
		   imagefilledpolygon($image, $points, 4, $color);
		   return imagepolygon($image, $points, 4, $color);
		}
	}