<?php
/*
Plugin Name: Rubik's Cube
Plugin URI: http://cube.caribmedia.com/tools/wordpress-rubik-plugin/
Description: A filter for WordPress that displays animated Rubik's cube algorithms
Version: 0.1
Author: Michiel van der Blonk
Author URI: http://www.vanderblonk.com
*/

/* replace [rubik alg=... type=... width=... height=...]
* by either
* <iframe ...> when type=iframe, results in an iframe loaded from this domain
* <object ...> when type=applet, results in a RubikPlayer applet taken from the local installed applet
* <image ...> when type=img, results in an ImageCube png image
*/

function debugVar ($s)
{
	return;
	echo '<textarea rows="10" cols="40">';
	print_r($s);
	echo '</textarea><br />';
}

class RubikPlugin {
		var $regexp;
		var $templates;
		var $iframeSrc;
		var $params; // not used yet
		var $templateNames;

    /**
     * Constructor: returns the object with some variables preset
     * @access   public
     * @return   object
     */
		function RubikPlugin() {
			// other stuff
			$this->regexp = "/\[rubik [^\]]+\]/";

			$this->validAttributes = array(
				/* default */ 	'width', 'height', 'type', 'alg',
				/* applet */	'code', 'archive', 'scriptLanguage', 'displaylines', 'colorTable', 'scriptType', 'bgcolor', 'faces', 'initscript', 'stickers', 'mode',
				/* animcube */ 'initrevmove', 'position', 'movetext', 'metric', 'fonttype', 'speed', 'doublespeed', 'perspective', 'scale', 'align', 'hint', 'edit', 'buttonbar', 'colorscheme',
				/* image */	'stickers', 'format', 'id',
				/* link */ 'inline'
			);
			$this->templateNames = array("iframe", "applet", "image", "link", "popup", "transformer", "transformer-with-close",
				"applet-animcube", "applet-caesar", "applet-rubikplayer", "applet-revenge", "applet-professor" );
			$this->iframeSrc = 'http://thearufam.brinkster.net/cube/cubeapplet.asp';
			foreach ($this->templateNames as $name)
				$this->templates[$name] = $this->getFileContents(dirname(__FILE__) . '/templates/' . $name . '.html');
		}

    /**
     * Replace all tags matching the [rubik ] pattern by their corresponding html
     * @param   string	$content	The actual article or comment content
     * @access   public
     * @return   string	the content with all [rubik ..] tags replaced by proper html
     */
		function parse($content) {
			$tags = array();
			$rubikAlgs = array();
			// get rubik tags
			preg_match_all($this->regexp, $content, &$rubikAlgs);
			$match = $rubikAlgs[0];
			// get attributes for each algorithm,
			// and store in list of Tag objects
			foreach ($match as $key=>$code)
			{
				$attribs = array();
				$tag = new StdClass();
				$tag->tagsrc = $code;
				// defaults
				$tag->type = 'applet';
				$tag->width = '200';
				$tag->height = '250';
				$tag->alg = '';
				$tag->scriptLanguage = 'HarrisENG';
				$tag->displaylines = 1;
				$tag->colorTable = 'b=0x003373,o=0xff4600,w=0xf8f8f8,g=0x00732f,r=0x8c000f,y=0xffd200,x=0x707070';
				$tag->scriptType = 'Solver';
				$tag->bgcolor = '0xFFFFFF';
				$tag->faces = 'o,b,w,r,g,y';
				$tag->code = 'ch.randelshofer.rubik.RubikPlayerApp.class';
				$tag->archive = '/wp-content/plugins/rubik/applets/rubikplayer-5-2-1.jar';
				$tag->stickers = 'rrrrrrrrrgggggggggwwwwwwwwwooooooooobbbbbbbbbyyyyyyyyy';
				$tag->format = 'png';
				// animcube
				$tag->position = '';
				$tag->movetext = '0';
				$tag->initrevmove = '';
				$tag->metric = "f";
				$tag->fonttype = '1';
				$tag->speed = '10';
				$tag->doublespeed = '10';
				$tag->perspective = '2'; // normal
				$tag->scale = '0'; // fit
				$tag->align = '1'; // center
				$tag->hint = '';
				$tag->edit = '1';
				$tag->buttonbar = '1';
				$tag->colorscheme = "WYORGB"; // UDFBLR

				foreach ($this->validAttributes as $param) {
					preg_match("/${param}=\"[^\"]+\"/", $code, &$paramOut);
					// debugVar($code);
					// check if we have a match and then read attributes
					if (!empty($paramOut[0]))
					{
						// split attribute name and value
						$attr = explode('=', $paramOut[0]);
						// check if we have a match and store in current tag object
						if (!empty($attr[0])) {
							$tag->$attr[0] = str_replace('"', '', $attr[1]);
							// store in array as well for easy replacing
							$attribs[$attr[0]] = $tag->$attr[0];
						}
					}
				}

				// fix for older style type='generator' and type='solver'
				if (strtolower($tag->type)=='generator' ||
					 strtolower($tag->type)=='solver')
					$tag->type = 'applet';
				
				// fix for older style type='stickers' instead of 'mode'
				if (strtolower($tag->stickers)=='cross' ||
					 strtolower($tag->stickers)=='f2l' ||
					 strtolower($tag->stickers)=='ll' ||
					 strtolower($tag->stickers)=='fl')
					$tag->mode = $tag->stickers;
				
				if ($tag->type=='image' || $tag->type=='transformer' || $tag->type=='transformer-with-close') {
					// correct jpg to jpeg
					if ($tag->format=='jpg')
						$tag->format='jpeg';
					// first check if image creation is being supported
					$ok = function_exists('imagecreate');
					$ok &= function_exists('imageCreateTrueColor');
					$ok &= function_exists('imageCopyResampled');
					$ok &= function_exists('imageColorAllocate');
					$ok &= function_exists('imageColorTransparent');
					$ok &= function_exists('imageFill');
					$ok &= function_exists('image' . strtolower($tag->format));
					if (!$ok)
						$this->showError("warning: GD image library not configured correctly.");
					else
					{
						require_once('images/Cube.php');
						$cube = new Cube;
						$cube->performAlg($tag->alg);
						$tag->stickers = $cube->toImageCubeStickers();
					}
				}
				if (eregi('applet', $tag->type)) {
						require_once('images/Cube.php');
						$cube = new Cube;
						if ($tag->mode)
								$cube->setMode($tag->mode);
						$tag->stickers = $cube->toAppletStickers(false, ',');
				}
				if ($tag->type=='applet-revenge' || $tag->type=='applet-professor')
					$tag->scriptLanguage = "SupersetENG";
				if ($tag->type=='link') {
					if (!$tag->inline)
						$tag->inline = "false";
				}
				$html = $this->templates[$tag->type];
				// replace attributes in template
				foreach ($this->validAttributes as $name)
					$html = str_replace("@${name}", $tag->$name, $html);
				$tag->html = $html;
				// add to list of Tag objects
				$tags[] = $tag;
			}
			// start replacing templates in original content
			$out = $content;
			foreach ($tags as $tag)
				$out = str_replace($tag->tagsrc, $tag->html, $out);
			return $out;
		}

	function getFileContents($s)
	{
		if (file_exists($s))
			return file_get_contents($s);
	}

	function showError($s)
	{
		echo '<p style="color: red" class="plugin-error">' . $s . '</p>';
	}

	function testPlugin()
	{
		$s = 'This is a text without an algorithm';
//		echo rubik_plugin($s) . "\n";
		$s = 'This is a text with one algorithm [rubik type="applet" alg="RL" width="340"] and some more text.';
//		echo rubik_plugin($s) . "\n";
		$s = 'This is a text with two algorithms [rubik type="applet" alg="RL" width="340"] and some more text [rubik type="applet" alg="RL" width="340"]  and some more text';
//		echo rubik_plugin($s) . "\n";
		$s = 'This is a text with two algorithms [rubik type="applet" alg="RL" width="340"] of different type [rubik type="image" alg="RL" width="340"]  and some more text';
//		echo rubik_plugin($s) . "\n";
		$s = 'This is a text with two algorithms without optional arguments [rubik alg="RL"] and [rubik alg="R\'U\'RU" width="340"]  and some more text';
//		echo rubik_plugin($s) . "\n";
		$s = 'This is a text with an AnimCube [rubik alg="RL" type="applet-animcube"]';
		echo rubik_plugin($s) . "\n";
		$s = 'This is a text with a Revenge [rubik alg="RL" type="applet-revenge"]';
		echo rubik_plugin($s) . "\n";
	}
}

function rubik_plugin($content) {
	$p = new RubikPlugin;
	debugVar($p);
	$output = $p->parse($content);
	debugVar($output);
	return $output;
}

// making sure this works outside of WordPress
if (function_exists('remove_filter'))
	remove_filter('the_content', 'wptexturize'); // disable smart quotes, these messup the code

if (function_exists('add_filter'))
{
	add_filter('the_content', 'rubik_plugin');
	add_filter('comment_text', 'rubik_plugin');
}

//RubikPlugin::testPlugin();
?>
